from sqlmodel import SQLModel, Field, Relationship
from typing import Optional, List
from datetime import datetime


class User(SQLModel, table=True):
    id: Optional[int] = Field(default=None, primary_key=True)
    username: str
    email: str = Field(unique=True, index=True)
    password: str
    role: str = Field(default="customer")  # Roles: customer, shop_owner, admin
    created_at: datetime = Field(default_factory=datetime.utcnow)

    shops: List["Shop"] = Relationship(back_populates="owner")
    orders: List["Order"] = Relationship(back_populates="user")


class Shop(SQLModel, table=True):
    id: Optional[int] = Field(default=None, primary_key=True)
    owner_id: int = Field(foreign_key="user.id")
    name: str = Field(unique=True, index=True)
    description: Optional[str] = None
    image_url: Optional[str] = None  # Image URL for shop
    # location coordinates
    address: str
    latitude: float
    longitude: float
    created_at: datetime = Field(default_factory=datetime.utcnow)

    owner: User = Relationship(back_populates="shops")
    products: List["Product"] = Relationship(back_populates="shop")


class Category(SQLModel, table=True):
    id: Optional[int] = Field(default=None, primary_key=True)
    name: str = Field(unique=True, index=True)

    products: List["Product"] = Relationship(back_populates="category")


class Product(SQLModel, table=True):
    id: Optional[int] = Field(default=None, primary_key=True)
    shop_id: int = Field(foreign_key="shop.id")
    category_id: int = Field(foreign_key="category.id", nullable=True)
    name: str
    description: Optional[str] = None
    price: float
    stock: int
    created_at: datetime = Field(default_factory=datetime.utcnow)

    shop: "Shop" = Relationship(back_populates="products")
    category: Optional["Category"] = Relationship(back_populates="products")
    images: List["ProductImage"] = Relationship(back_populates="product")


class ProductImage(SQLModel, table=True):
    id: Optional[int] = Field(default=None, primary_key=True)
    product_id: int = Field(foreign_key="product.id")
    image_url: str = Field(nullable=False)  # Store each image URL separately

    product: Product = Relationship(back_populates="images")


class Order(SQLModel, table=True):
    id: Optional[int] = Field(default=None, primary_key=True)
    user_id: int = Field(foreign_key="user.id")
    shop_id: int = Field(foreign_key="shop.id")
    total_price: float
    shipping_price: float
    status: str = Field(default="pending")
    # delivery location coordinates
    delivery_address: str
    delivery_latitude: float
    delivery_longitude: float
    created_at: datetime = Field(default_factory=datetime.utcnow)

    user: User = Relationship(back_populates="orders")
    order_items: List["OrderItem"] = Relationship(back_populates="order")


class OrderItem(SQLModel, table=True):
    id: Optional[int] = Field(default=None, primary_key=True)
    order_id: int = Field(foreign_key="order.id")
    product_id: int = Field(foreign_key="product.id")
    quantity: int
    price: float

    order: Order = Relationship(back_populates="order_items")
